function tones = decolorizefile(inputfile,outputfile,effect,scale,noise,varargin)
% Created: 4 June 2005
% Version: 16 July 2009
% Copyright 2009 Mark Grundland. All rights resaved. 
% Licensed for noncommercial, academic use only.

% Developed by Mark Grundland.
% Computer Laboratory, University of Cambridge, UK.
% Technical Report UCAM-CL-TR-649, University of Cambridge.
% Web page: http://www.eyemaginary.com/Portfolio/Publications.html
% Any questions? Contact mark @ eyemaginary.com .

% Usage:   Coverts RGB picture to grayscale tones,
%             while endeavoring to express color contrasts as tone changes.
% Params:  inputfile specifies the input file
%             should include the file's directory, name, and format
%          outputfile specifies the output file
%             should include the file's directory, name, and format
% Options: effect specifies how much the picture's achromatic content 
%             should be altered to accommodate the chromatic contrasts
%             (default effect = 0.5 )
%          scale in pixels is the typical size of 
%             relevant color contrast features
%             (default scale = sqrt(2*min(size(picture.dimensions)) )
%          noise quantile indicates the amount of noise in the picture
%             enabling the dynamic range of the tones to be appropriately scaled
%             (default noise = 0.001)
%          additional file format save options for imwrite may be appended
%             (see MatLab Help: imwrite(...,Param1,Val1,Param2,Val2...) )
% Assumes: valid RGB 0 <= picture <= 1
%          positive 0 <= effect <= 1  
%          positive 1 <= scale << min(picture.dimensions)
%          small quantile 0 <= noise << 0.5 
% Applies: Standard NTSC color to grayscale conversion
%             is used as the default achromatic color channel.
% Example: decolorizefile("Input.JPEG","Output.JPEG",0.5,25,0.001,'Quality',90)


% Read the image file
if nargin<1 || isempty(inputfile) || ~exist(inputfile,'file')
    [inputfile, path]=uigetfile(...
            {'*.bmp;*.cur;*.hdf;*.ico;*.jpg;*.jpeg;*.pcx;*.png;*.tif;*.tiff;*.xwd;*.mat','Image Files'},...
            'Open the source image file:');
    if length(inputfile)==1 && inputfile==0
        tones=[];
        return;
    end;
    inputfile=[path filesep inputfile];
end;
picture=imread(inputfile);
picture=im2double(picture);
picture=single(picture);

% Examine inputs
frame=[size(picture,1), size(picture,2)];
pixels=frame(1)*frame(2);
if nargin<2 || isempty(outputfile)
    [path, outputfile, format] = fileparts(inputfile);
    outputfile=[path filesep outputfile '-Gray' format];
end;
if nargin<3 || isempty(effect)
    effect=0.5;
end;
if nargin<4 || isempty(scale)
    scale=sqrt(2*min(frame));
end;
if nargin<5 || isempty(noise)
    noise=0.001;
end;

% Convert to grayscale
tones=decolorizeimage(picture,effect,scale,noise);

% Write the image file
imwrite(tones,outputfile,varargin{:});
